import sys

from ingenialink.ethercat.network import EthercatNetwork


def connect_slave():
    net = EthercatNetwork("\\Device\\NPF_{192D1D2F-C684-467D-A637-EC07BD434A63}")
    servo = net.connect_to_slave(
        target=1,
        dictionary='../../resources/dictionaries/cap-net-e_eoe_0.7.1.xdf')
    return servo, net


def store_restore_example():
    """Connects to the first scanned drive and store and restores the
    current configuration."""

    servo, net = connect_slave()

    # Store all
    try:
        servo.store_parameters()
        print('Stored all parameters successfully')
    except Exception as e:
        print('Error storing all parameters')

    # Store axis 1
    try:
        servo.store_parameters(subnode=1)
        print('Stored axis 1 parameters successfully')
    except Exception as e:
        print('Error storing parameters axis 1.')

    # Restore all
    try:
        servo.restore_parameters()
        print('Restored all parameters successfully')
    except Exception as e:
        print('Error restoring all parameters')

    net.disconnect_from_slave(servo)


if __name__ == '__main__':
    store_restore_example()
    sys.exit()



import pytest

from ingenialink.ipb.dictionary import IPBDictionary
from ingenialink.ipb.register import IPBRegister
from ingenialink.register_deprecated import REG_DTYPE, REG_ACCESS, REG_PHY


@pytest.mark.ethercat
def test_ipb_register(connect_to_slave):
    servo, net = connect_to_slave
    assert servo is not None and net is not None

    assert isinstance(servo.dictionary, IPBDictionary)

    registers_sub_0 = servo.dictionary.registers(0)
    assert registers_sub_0
    registers_sub_1 = servo.dictionary.registers(1)
    assert registers_sub_1

    register = registers_sub_1.get("DRV_OP_CMD")
    assert isinstance(register, IPBRegister)

    assert register.identifier == "DRV_OP_CMD"
    assert register.units == "-"
    assert register.cyclic == "CYCLIC_RX"
    assert register.dtype == REG_DTYPE.U16
    assert register.access, REG_ACCESS.RW
    assert register.address == 0x0014
    assert register.phy == REG_PHY.NONE
    assert register.subnode == 1
    assert register.storage is None
    assert register.storage_valid == 0
    assert register.range == (0, 65535)
    assert register.labels is not None
    assert len(register.enums) == 13
    assert register.enums_count == 13
    assert register.cat_id == "TARGET"
    assert register.scat_id is None
    assert register.internal_use == 0



try:
    from ._ingenialink import lib
except ImportError as e:
    raise ImportError("DLLs required not found: Please install WinPcap") from e

from .network import NET_PROT, NET_STATE, NET_DEV_EVT, NET_TRANS_PROT, Network, EEPROM_FILE_FORMAT
from ingenialink.enums.servo import (
    SERVO_STATE,
    SERVO_FLAGS,
    SERVO_MODE,
    SERVO_UNITS_TORQUE,
    SERVO_UNITS_POS,
    SERVO_UNITS_VEL,
    SERVO_UNITS_ACC,
)
from ingenialink.servo import Servo

from .ipb.poller import IPBPoller
from .ipb.register import IPBRegister
from .ipb.dictionary import IPBDictionary
from .ipb.network import NetworkMonitor

from .ethernet.network import EthernetNetwork
from .ethernet.servo import EthernetServo

from .ethercat.network import EthercatNetwork
from .ethercat.servo import EthercatServo

from .canopen.servo import CanopenServo
from .canopen.network import CanopenNetwork, CAN_DEVICE, CAN_DEVICE, CAN_BAUDRATE
from .canopen.register import CanopenRegister
from .canopen.dictionary import CanopenDictionary

from ingenialink.enums.register import REG_DTYPE, REG_ACCESS, REG_PHY

from ingenialink.utils.errors import err_ipb_last

from ingenialink.utils._utils import pstr, set_logger_level

from ingenialink.poller import Poller

set_logger_level(3)

__all__ = [
    "EEPROM_FILE_FORMAT",
    "NET_PROT",
    "NET_DEV_EVT",
    "NET_STATE",
    "NET_TRANS_PROT",
    "SERVO_STATE",
    "SERVO_FLAGS",
    "SERVO_MODE",
    "SERVO_UNITS_TORQUE",
    "SERVO_UNITS_POS",
    "SERVO_UNITS_VEL",
    "SERVO_UNITS_ACC",
    "NetworkMonitor",
    "Network",
    "Servo",
    "IPBDictionary",
    "IPBRegister",
    "REG_DTYPE",
    "REG_ACCESS",
    "REG_PHY",
    "IPBPoller",
    "EthercatNetwork",
    "EthercatServo",
    "EthernetServo",
    "EthernetNetwork",
    "CanopenNetwork",
    "CAN_DEVICE",
    "CAN_BAUDRATE",
    "CanopenServo",
    "CanopenRegister",
    "Poller",
    "CanopenDictionary",
    "err_ipb_last",
]

__version__ = "6.5.1"

try:
    __ingenialink_C_version__ = pstr(lib.il_version())
except Exception:
    __ingenialink_C_version__ = "-"



import sys

from ingenialink.ethercat.network import EthercatNetwork


def connect_slave():
    net = EthercatNetwork("\\Device\\NPF_{192D1D2F-C684-467D-A637-EC07BD434A63}")
    servo = net.connect_to_slave(
        target=1,
        dictionary='../../resources/dictionaries/cap-net-e_eoe_0.7.1.xdf')
    return servo, net


def load_config_example():
    """Loads a given configuration file into the drive."""
    servo, net = connect_slave()
    servo.load_configuration('ecat_config.xcf')
    servo.load_configuration('ecat_config_0.xcf', subnode=0)
    servo.load_configuration('ecat_config_1.xcf', subnode=1)

    net.disconnect_from_slave(servo)


def save_config_example():
    """Saves the drive configuration into a file."""
    servo, net = connect_slave()
    servo.save_configuration('ecat_config.xcf')
    servo.save_configuration('ecat_config_0.xcf', subnode=0)
    servo.save_configuration('ecat_config_1.xcf', subnode=1)

    net.disconnect_from_slave(servo)


if __name__ == '__main__':
    save_config_example()
    load_config_example()
    sys.exit()



import pytest

from ingenialink.dictionary_deprecated import Dictionary


@pytest.mark.ethercat
def test_dictionary(connect_to_slave):
    servo, net = connect_to_slave
    assert servo is not None and net is not None

    assert isinstance(servo.dictionary, Dictionary)

    assert servo.dictionary.path is not None
    assert servo.dictionary.version is not None
    assert servo.dictionary.firmware_version is not None
    assert servo.dictionary.revision_number is not None
    assert servo.dictionary.product_code is not None
    assert servo.dictionary.interface is not None
    assert servo.dictionary.part_number is not None
    assert servo.dictionary.subnodes is not None
    assert servo.dictionary.errors is not None
    assert servo.dictionary.categories is not None
    assert servo.dictionary.registers(0) is not None
    assert servo.dictionary.registers(1) is not None



import pytest

from ingenialink.register import Register
from ingenialink.register import REG_DTYPE, REG_ACCESS, REG_PHY, dtypes_ranges
from ingenialink.utils._utils import exc


@pytest.mark.no_connection
def test_getters_register():
    reg_dtype = REG_DTYPE.U32
    reg_access = REG_ACCESS.RW
    reg_kwargs = {
        "identifier": "MON_CFG_SOC_TYPE",
        "units": "none",
        "cyclic": "CONFIG",
        "phy": REG_PHY.NONE,
        "subnode": 0,
        "storage": 1,
        "reg_range": (-20, 20),
        "labels": "Monitoring trigger type",
        "enums": {"0": "TRIGGER_EVENT_AUTO", "1": "TRIGGER_EVENT_FORCED"},
        "cat_id": "MONITORING",
        "scat_id": "SUB_CATEGORY_TEST",
        "internal_use": "No description (invent here)",
    }
    aux_enums = []
    for key, value in reg_kwargs["enums"].items():
        test_dictionary = {"label": value, "value": int(key)}
        aux_enums.append(test_dictionary)

    register = Register(reg_dtype, reg_access, **reg_kwargs)

    assert register.identifier == reg_kwargs["identifier"]
    assert register.units == reg_kwargs["units"]
    assert register.cyclic == reg_kwargs["cyclic"]
    assert register.dtype == reg_dtype
    assert register.access == reg_access
    assert register.phy == reg_kwargs["phy"]
    assert register.subnode == reg_kwargs["subnode"]
    assert register.storage == reg_kwargs["storage"]
    assert register.range == reg_kwargs["reg_range"]
    assert register.labels == reg_kwargs["labels"]
    assert register.cat_id == reg_kwargs["cat_id"]
    assert register.scat_id == reg_kwargs["scat_id"]
    assert register.internal_use == reg_kwargs["internal_use"]
    assert register.enums == aux_enums
    assert register.enums_count == 2
    assert register.storage_valid == True


@pytest.mark.no_connection
def test_register_type_errors():
    dtype = "False type"
    access = REG_ACCESS.RW
    with pytest.raises(exc.ILValueError):
        Register(dtype, access)

    dtype = REG_DTYPE.FLOAT
    access = "False access"
    with pytest.raises(exc.ILAccessError):
        Register(dtype, access)

    dtype = REG_DTYPE.FLOAT
    access = REG_ACCESS.RW
    with pytest.raises(exc.ILValueError):
        Register(dtype, access, phy="False Phy")


@pytest.mark.no_connection
def test_register_get_storage():
    access = REG_ACCESS.RW

    # invalid storage
    dtype = REG_DTYPE.STR
    register = Register(dtype, access, storage=1)
    assert register.storage_valid == 0
    assert register.storage is None

    # no storage
    dtype = REG_DTYPE.FLOAT
    register = Register(dtype, access)
    assert register.storage_valid == 0
    assert register.storage is None

    # float storage
    dtype = REG_DTYPE.FLOAT
    storage = 12.34
    register = Register(dtype, access, storage=storage)
    assert register.storage_valid == 1
    assert register.storage == storage

    # parse float storage
    dtype = REG_DTYPE.FLOAT
    storage = 123
    register = Register(dtype, access, storage=storage)
    assert type(register.storage) is float

    # parse int storage
    dtype = REG_DTYPE.U8
    storage = 123.1
    register = Register(dtype, access, storage=storage)
    assert type(register.storage) is int
    assert register.storage == 123


@pytest.mark.no_connection
def test_register_set_storage():
    access = REG_ACCESS.RW
    dtype = REG_DTYPE.FLOAT
    storage = 20.0
    register = Register(dtype, access, storage=storage)
    assert register.storage == storage

    storage = 1.1
    register.storage = storage
    assert register.storage == storage


@pytest.mark.no_connection
def test_register_range():
    access = REG_ACCESS.RW

    # custom range
    dtype = REG_DTYPE.U8
    range = (0, 100)
    register = Register(dtype, access, reg_range=range)
    assert type(register.range[0]) is int
    assert type(register.range[1]) is int
    assert register.range == range

    # custom range float
    dtype = REG_DTYPE.FLOAT
    range = (1.11, 100.25)
    register = Register(dtype, access, reg_range=range)
    assert type(register.range[0]) is float
    assert type(register.range[1]) is float
    assert register.range == range

    # default range
    dtype = REG_DTYPE.U8
    register = Register(dtype, access)
    assert register.range == (dtypes_ranges[dtype]["min"], dtypes_ranges[dtype]["max"])



import sys

from ingenialink.ethercat.network import EthercatNetwork


def connection_example():
    net = EthercatNetwork("\\Device\\NPF_{13C5D891-C81E-46CE-8651-FADBE3C9415D}")

    slaves = net.scan_slaves()
    print(slaves)

    if len(slaves) > 0:
        servo = net.connect_to_slave(
            target=slaves[0],
            dictionary='../../resources/dictionaries/eve-xcr-e_eoe_1.8.1.xdf')

        print(servo.read('DRV_ID_SOFTWARE_VERSION'))
        net.disconnect_from_slave(servo)


if __name__ == '__main__':
    connection_example()
    sys.exit(0)



from time import sleep
from threading import Thread
from abc import ABC, abstractmethod
from enum import Enum
from .._ingenialink import lib, ffi
from ingenialink.exceptions import ILError
from ingenialink.utils._utils import pstr, raise_err, raise_null

import ingenialogger

logger = ingenialogger.get_logger(__name__)


class NET_PROT(Enum):
    """Network Protocol."""

    EUSB = lib.IL_NET_PROT_EUSB
    MCB = lib.IL_NET_PROT_MCB
    ETH = lib.IL_NET_PROT_ETH
    ECAT = lib.IL_NET_PROT_ECAT
    CAN = 5


class NET_STATE(Enum):
    """Network State."""

    CONNECTED = lib.IL_NET_STATE_CONNECTED
    DISCONNECTED = lib.IL_NET_STATE_DISCONNECTED
    FAULTY = lib.IL_NET_STATE_FAULTY


class NET_DEV_EVT(Enum):
    """Device Event."""

    ADDED = lib.IL_NET_DEV_EVT_ADDED
    REMOVED = lib.IL_NET_DEV_EVT_REMOVED


@ffi.def_extern()
def _on_found_cb(ctx, servo_id):
    """On found callback shim."""
    self = ffi.from_handle(ctx)
    self._on_found(int(servo_id))


@ffi.def_extern()
def _on_evt_cb(ctx, evt, port):
    """On event callback shim."""
    self = ffi.from_handle(ctx)
    self._on_evt(NET_DEV_EVT(evt), pstr(port))


class NetStatusListener(Thread):
    """Network status listener thread to check if the drive is alive.

    Args:
        network (IPBNetwork): Network instance of the IPB communication.

    """

    def __init__(self, network):
        super(NetStatusListener, self).__init__()
        self.__net = network
        self.__stop = False

    def run(self):
        status = self.__net.status
        while not self.__stop:
            if status != self.__net.status:
                if self.__net.status == NET_STATE.CONNECTED.value:
                    self.__net._notify_status(NET_DEV_EVT.ADDED)
                elif self.__net.status == NET_STATE.DISCONNECTED.value:
                    self.__net._notify_status(NET_DEV_EVT.REMOVED)
                status = self.__net.status
            sleep(1)

    def stop(self):
        self.__stop = True


class IPBNetwork(ABC):
    """IPB Network defines a general class for all IPB based communications."""

    def __init__(self):
        super(IPBNetwork, self).__init__()
        self.servos = []
        self._cffi_network = None
        """CFFI instance of the network."""

        self.__observers_net_state = []
        self.__listener_net_status = None

    def _create_cffi_network(self, cffi_network):
        """Create a new class instance from an existing network.

        Args:
            cffi_network (CData): Instance to copy.

        """
        self._cffi_network = ffi.gc(cffi_network, lib.il_net_fake_destroy)

    @abstractmethod
    def scan_slaves(self):
        raise NotImplementedError

    @abstractmethod
    def connect_to_slave(self, *args, **kwargs):
        raise NotImplementedError

    @abstractmethod
    def disconnect_from_slave(self, servo):
        raise NotImplementedError

    @abstractmethod
    def load_firmware(self, *args, **kwargs):
        raise NotImplementedError

    def close_socket(self):
        """Closes the established network socket."""
        return lib.il_net_close_socket(self._cffi_network)

    def destroy_network(self):
        """Destroy network instance."""
        lib.il_net_destroy(self._cffi_network)

    def subscribe_to_status(self, callback):
        """Calls given function everytime a connection/disconnection event is
        raised.

        Args:
            callback (function): Function that will be called every time an event
                is raised.

        """
        if callback in self.__observers_net_state:
            logger.info("Callback already subscribed.")
            return
        self.__observers_net_state.append(callback)

    def unsubscribe_from_status(self, callback):
        """Unsubscribe from state changes.

        Args:
            callback (function): Callback function.

        """
        if callback not in self.__observers_net_state:
            logger.info("Callback not subscribed.")
            return
        self.__observers_net_state.remove(callback)

    def _notify_status(self, status):
        for callback in self.__observers_net_state:
            callback(status)

    def _set_status_check_stop(self, stop):
        """Start/Stop the internal monitor of the drive status.

        Args:
            stop (int): 0 to START, 1 to STOP.

        Raises:
            ILError: If the operation returns a negative error code.

        """
        r = lib.il_net_set_status_check_stop(self._cffi_network, stop)

        if r < 0:
            raise ILError("Could not start servo monitoring")

    def start_status_listener(self):
        """Start monitoring network events (CONNECTION/DISCONNECTION)."""
        self._set_status_check_stop(0)
        if not self.__listener_net_status:
            self.__listener_net_status = NetStatusListener(self)
            self.__listener_net_status.start()

    def stop_status_listener(self):
        """Stop monitoring network events (CONNECTION/DISCONNECTION)."""
        self._set_status_check_stop(1)
        if self.__listener_net_status is not None and self.__listener_net_status.is_alive():
            self.__listener_net_status.stop()
            self.__listener_net_status.join()
        self.__listener_net_status = None

    def set_reconnection_retries(self, retries):
        """Set the number of reconnection retries in our application.

        Args:
            retries (int): Number of reconnection retries.

        """
        return lib.il_net_set_reconnection_retries(self._cffi_network, retries)

    def set_recv_timeout(self, timeout):
        """Set receive communications timeout.

        Args:
            timeout (int): Timeout in ms.
        Returns:
            int: Result code.

        """
        return lib.il_net_set_recv_timeout(self._cffi_network, timeout * 1000)

    @property
    def protocol(self):
        raise NotImplementedError

    @property
    def status(self):
        """NET_STATE: Obtain network status"""
        return lib.il_net_status_get(self._cffi_network)


class NetworkMonitor:
    """Network Monitor.

    Args:
        prot (NET_PROT): Protocol.

    Raises:
        TypeError: If the protocol type is invalid.
        ILCreationError: If the monitor cannot be created.

    """

    def __init__(self, prot):
        if not isinstance(prot, NET_PROT):
            raise TypeError("Invalid protocol")

        mon = lib.il_net_dev_mon_create(prot.value)
        raise_null(mon)

        self._mon = ffi.gc(mon, lib.il_net_dev_mon_destroy)

    def start(self, on_evt):
        """Start the monitor.

        Args:
            on_evt (callback): Callback function.

        """
        self._on_evt = on_evt
        self._handle = ffi.new_handle(self)

        r = lib.il_net_dev_mon_start(self._mon, lib._on_evt_cb, self._handle)
        raise_err(r)

    def stop(self):
        """Stop the monitor."""
        lib.il_net_dev_mon_stop(self._mon)



import pytest

from ingenialink.ethercat.network import EthercatNetwork
from ingenialink.exceptions import ILFirmwareLoadError


@pytest.mark.ethercat
def test_connect_to_slave(connect_to_slave):
    servo, net = connect_to_slave
    assert servo is not None and net is not None
    assert len(net.servos) == 1
    fw_version = servo.read("DRV_ID_SOFTWARE_VERSION")
    assert fw_version is not None and fw_version != ""


@pytest.mark.ethercat
def test_scan_slaves(read_config):
    net = EthercatNetwork(read_config["ethercat"]["ifname"])

    slaves = net.scan_slaves()
    assert len(slaves) > 0


@pytest.mark.ethercat
def test_load_firmware_file_not_found(read_config):
    net = EthercatNetwork(read_config["ethercat"]["ifname"])
    with pytest.raises(FileNotFoundError):
        net.load_firmware(fw_file="ethercat.sfu")


@pytest.mark.ethercat
def test_load_firmware_load_error(mocker, read_config):
    net = EthercatNetwork(read_config["ethercat"]["ifname"])
    mocker.patch("os.path.isfile", return_value=True)
    with pytest.raises(ILFirmwareLoadError):
        net.load_firmware(fw_file="ethercat.sfu")


@pytest.mark.ethercat
def test_load_firmware_value_error(mocker, read_config):
    net = EthercatNetwork(read_config["ethercat"]["ifname"])
    mocker.patch("os.path.isfile", return_value=True)
    with pytest.raises(ValueError):
        net.load_firmware(fw_file="ethercat.xyz")



import sys
import re
import os
from os.path import join, exists
from distutils.spawn import find_executable
from subprocess import check_call

from cffi import FFI


_SRC_DIR = "_deps"
_BUILD_DIR = "_build"
_INSTALL_DIR = "_install"

_INC_DIR = join(_INSTALL_DIR, "include")
_LIB_DIR = join(_INSTALL_DIR, "lib")

if "INGENIALINK_DIR" in os.environ:
    _IL_URL = None
    _IL_SRC = os.environ["INGENIALINK_DIR"]
else:
    _IL_URL = "https://github.com/ingeniamc/ingenialink"
    _IL_VER = "6.3.2"
    _IL_SRC = join(_SRC_DIR, "ingenialink")

_IL_BUILD = join(_BUILD_DIR, "ingenialink")

if "XML2_DIR" in os.environ:
    _XML2_SRC = os.environ["XML2_DIR"]
else:
    _XML2_SRC = join(_IL_SRC, "external", "libxml2")

_XML2_BUILD = join(_BUILD_DIR, "libxml2")

if "SOEM_DIR" in os.environ:
    _SOEM_SRC = os.environ["SOEM_DIR"]
else:
    _SOEM_SRC = join(_IL_SRC, "external", "SOEM")

_SOEM_BUILD = join(_BUILD_DIR, "soem")

if sys.platform == "win32":
    if sys.version_info >= (3, 5):
        _CMAKE_GENERATOR = "Visual Studio 14 2015"
    else:
        raise ImportError("Unsupported Python version")

    if sys.maxsize > 2**32:
        _CMAKE_GENERATOR += " Win64"
else:
    _CMAKE_GENERATOR = "Unix Makefiles"


def _build_deps():
    """Obtains and build dependencies."""

    # Check for Git & CMake
    git = find_executable("git")
    if not git:
        raise FileNotFoundError("Git is not installed or in PATH")

    cmake = find_executable("cmake")
    if not cmake:
        raise FileNotFoundError("CMake is not installed or in PATH")

    # Clone
    if not exists(_IL_SRC) and _IL_URL:
        check_call([git, "clone", "--recursive", "-b", _IL_VER, _IL_URL, _IL_SRC])

    check_call(
        [
            cmake,
            "-H" + _XML2_SRC,
            "-B" + _XML2_BUILD,
            "-G",
            _CMAKE_GENERATOR,
            "-DCMAKE_BUILD_TYPE=Release",
            "-DCMAKE_INSTALL_PREFIX=" + _INSTALL_DIR,
            "-DBUILD_SHARED_LIBS=OFF",
            "-DWITH_PIC=ON",
        ]
    )
    check_call([cmake, "--build", _XML2_BUILD, "--config", "Release", "--target", "install"])

    check_call(
        [
            cmake,
            "-H" + _SOEM_SRC,
            "-B" + _SOEM_BUILD,
            "-G",
            _CMAKE_GENERATOR,
            "-DCMAKE_BUILD_TYPE=Release",
            "-DCMAKE_INSTALL_PREFIX=" + _INSTALL_DIR,
            "-DBUILD_SHARED_LIBS=OFF",
            "-DWITH_PIC=ON",
            "-DCMAKE_POSITION_INDEPENDENT_CODE=ON",
        ]
    )
    check_call([cmake, "--build", _SOEM_BUILD, "--config", "Release", "--target", "install"])

    print("[INFO] Ingenialink build")
    check_call(
        [
            cmake,
            "-H" + _IL_SRC,
            "-B" + _IL_BUILD,
            "-G",
            _CMAKE_GENERATOR,
            "-DCMAKE_BUILD_TYPE=Release",
            "-DCMAKE_INSTALL_PREFIX=" + _INSTALL_DIR,
            "-DBUILD_SHARED_LIBS=OFF",
            "-DWITH_PROT_ECAT=ON",
            "-DWITH_PROT_ETH=ON",
            "-DWITH_PIC=ON",
        ]
    )
    check_call([cmake, "--build", _IL_BUILD, "--config", "Release", "--target", "install"])


def _gen_cffi_header():
    """Generates cffi header.

    All ingenialink headers are joined into a single one, and, all
    cffi non-compatibe portions removed.

    Returns:
        str: cffi header.

    """
    remove = [
        "IL_EXPORT",
        "IL_BEGIN_DECL",
        "IL_END_DECL",
        "#ifdef.*",
        "#ifndef.*",
        "#endif.*",
        "#define PUBLIC.*",
        "#include.*",
        ".+foreach.+\n.*",
    ]

    headers = [
        join(_INC_DIR, "ingenialink", "const.h"),
        join(_INC_DIR, "ingenialink", "err.h"),
        join(_INC_DIR, "ingenialink", "dict_labels.h"),
        join(_INC_DIR, "ingenialink", "registers.h"),
        join(_INC_DIR, "ingenialink", "dict.h"),
        join(_INC_DIR, "ingenialink", "net.h"),
        join(_INC_DIR, "ingenialink", "servo.h"),
        join(_INC_DIR, "ingenialink", "poller.h"),
        join(_INC_DIR, "ingenialink", "version.h"),
    ]

    h_stripped = ""

    for header in headers:
        with open(header) as h:
            h_stripped += re.sub("|".join(remove), "", h.read())

    return h_stripped


def _get_libs():
    """Obtain the list of libraries to link against based on platform.

    Returns:
        list: List of libraries.

    """
    libs = ["ingenialink", "xml2", "soem"]

    if sys.platform.startswith("linux"):
        libs.extend(["udev", "rt", "pthread"])
    elif sys.platform == "darwin":
        libs.extend(["pthread"])
    elif sys.platform == "win32":
        libs.extend(
            ["user32", "setupapi", "advapi32", "wpcap", "ws2_32", "winmm", "gdi32", "dxguid"]
        )

    return libs


def _get_link_args():
    """Obtain the list of extra linker arguments based on platform.

    Returns:
        list: List of extra linker arguments.

    """
    if sys.platform == "darwin":
        return ["-framework", "IOKit", "-framework", "Foundation"]

    return []


# build dependencies first
_build_deps()

# cffi builder
ffibuilder = FFI()

ffibuilder.cdef(
    _gen_cffi_header()
    + """
    /* callbacks */
    extern "Python" void _on_found_cb(void *ctx, uint8_t node_id);
    extern "Python" void _on_evt_cb(void *ctx, il_net_dev_evt_t on_evt,
                                    const char *port);
    extern "Python" void _on_state_change_cb(void *ctx,
                                             il_servo_state_t state,
                                             int flags, uint8_t subnode);
    extern "Python" void _on_emcy_cb(void *ctx, uint32_t code);
"""
)

ffibuilder.set_source(
    "ingenialink._ingenialink",
    r"#include <ingenialink/ingenialink.h>",
    include_dirs=[_INC_DIR],
    library_dirs=[_LIB_DIR],
    libraries=_get_libs(),
    extra_link_args=_get_link_args(),
)


if __name__ == "__main__":
    ffibuilder.compile(verbose=True)
